from bunch import Bunch
from datetime import datetime
from mock import MagicMock

from blockerbugs import db
from blockerbugs.models.bug import Bug
from blockerbugs.models.milestone import Milestone
from blockerbugs.models.release import Release
from blockerbugs.models.update import Update
from blockerbugs.util.update_sync import UpdateSync

from testing.testfunc_bugmodel import generate_milestone, generate_bug, generate_update

templateupdate = Bunch(
    approved=None,
    close_bugs=True,
    critpath=False, critpath_approved=True, date_modified=None,
    date_pushed=u'2012-09-13 16:40:28', date_submitted=u'2012-09-12 23:43:04',
    karma=1,
    notes=u'Fix live install mounting /home and /boot and swap',
    request=None, stable_karma=3, status=u'testing', submitter=u'bcl',
    title=u'anaconda-18.6.8-1.fc18', type=u'bugfix', unstable_karma=-3,
    updateid=u'FEDORA-2012-13902',
    bugs=[Bunch(bz_id=856836, parent=False, security=False,
                title=u'/etc/fstab is not written correctly after live install (F18 Alpha RC2+)')],
    builds=[Bunch(nvr=u'anaconda-18.6.8-1.fc18',
                  package=Bunch(
                      committers=[u'anaconda-maint', u'dlehman', u'jkeating',
                                  u'bcl', u'clumens', u'rvykydal', u'akozumpl',
                                  u'pjones',
                                  u'katzj', u'dcantrel'],
                      name=u'anaconda', suggest_reboot=False))],
    comments=[
        Bunch(anonymous=False, author=u'bodhi', group=None, karma=0,
              text=u'This update has been submitted for testing by bcl. ',
              timestamp=u'2012-09-12 23:43:24',
              update_title=u'anaconda-18.6.8-1.fc18')
    ],
    nagged=Bunch(test_cases=[u'QA:Testcase Anaconda autopart install']),
    release=Bunch(dist_tag=u'f18', id_prefix=u'FEDORA', locked=True,
                  long_name=u'Fedora 18', name=u'F18'),
)


def generate_updateinfo(title, bugs, status='testing', pending=False,
                        date_testing=datetime(2012, 9, 12, 23, 43, 04),
                        date_submitted=datetime(2012, 9, 12, 23, 43, 04),
                        karma=1, stable_karma=3):
    return {'title': title,
            'karma': karma,
            'stable_karma': stable_karma,
            'status': status,
            'date_submitted': date_submitted,
            'date_pushed_testing': date_testing,
            'date_pushed_stable': None,
            'bugs': bugs,
            'url': 'https://admin.fedoraproject.org/updates/anaconda-18.6.8-1.fc18',
            'pending': pending}


class FakeBodhiRss(object):
    def __init__(self, bugnums):
        self.bugnums = bugnums

    def get_fixing_updates(self, release):
        return self.bugnums


class FakeBodhiInterface(object):
    def __init__(self, updateinfo):
        self.updateinfo = updateinfo

    def get_update(self, envr):
        print "I'm a mock get_update!"
        return self.updateinfo


class TestfuncUpdateSync(object):
    def setup_method(self, method):
        db.drop_all()
        db.create_all()
        test_release99 = Release(99)
        test_milestone99alpha = Milestone(test_release99, 'alpha', 567890,
                                          567891, '99-final')
        test_milestone99alpha.active = True
        db.session.add(test_release99)
        db.session.add(test_milestone99alpha)
        db.session.commit()

        self.testsync = UpdateSync(db, MagicMock())


    # def generate_bug(bugid, summary, release):
    # add 1 bug to db, mock out bodhi to return relevant update, check for entries
#    def test_add_single_bug_with_update(self, monkeypatch):
#
#        # set up bugs needed in db
#        test_milestone = Milestone.query.filter_by(active = True).first()
#        testbug1 = generate_bug(123456, 'first test bug', test_milestone)
#        db.session.add(testbug1)
#        db.session.commit()
#
#        # set up rss mock
#        testrss = FakeBodhiRss([testbug1.bugid])
#        monkeypatch.setattr(self.testsync, 'get_fixing_updates', testrss.get_fixing_updates)
#
#        # set up mock bodhi returns
#        testupdate1 = generate_updateinfo('magicfix-1.0-1.fc99', [testbug1.bugid])
#        testbodhi = FakeBodhiInterface(testupdate1)
#        monkeypatch.setattr(self.testsync, 'get_update', testbodhi.get_update)
#
#        self.testsync.sync_updates(test_milestone.release)
#
#        updates = Update.query.filter_by(title=testupdate1['title']).all()
#        assert len(updates) == 1
#        update = updates[0]
#        assert update.title == testupdate1['title']
#        assert len(update.bugs) == 1
#
#    # add 2 bugs to db, mock bodhi returns one relevent update, check for entries
#    def test_add_twobugs_one_update(self, monkeypatch):
#
#        # set up bugs needed in db
#        test_milestone = Milestone.query.filter_by(active = True).first()
#        testbug1 = generate_bug(123456, 'first test bug', test_milestone)
#        testbug2 = generate_bug(234567, 'second test bug', test_milestone)
#        db.session.add(testbug1)
#        db.session.add(testbug2)
#        db.session.commit()
#
#        # set up rss mock
#        testrss = FakeBodhiRss([testbug1.bugid])
#        monkeypatch.setattr(self.testsync, 'get_fixing_updates', testrss.get_fixing_updates)
#
#        # set up mock bodhi returns
#        testupdate1 = generate_updateinfo('magicfix-1.0-1.fc99', [testbug1.bugid])
#        testbodhi = FakeBodhiInterface(testupdate1)
#        monkeypatch.setattr(self.testsync, 'get_update', testbodhi.get_update)
#
#        self.testsync.sync_updates(test_milestone.release)
#
#        bug1 = Bug.query.filter_by(bugid=testbug1.bugid).first()
#        print bug1.updates
#        assert len(bug1.updates) == 1
#        assert bug1.updates[0].title == testupdate1['title']
#
#        bug2 = Bug.query.filter_by(bugid=testbug2.bugid).first()
#        assert len(bug2.updates) == 0
#
#    # add 1 bug to db, mock bodhi returns update w 2/ bugs, verify entries
#    def test_add_onebug_one_update_twofixes(self, monkeypatch):
#
#        # set up bugs needed in db
#        test_milestone = Milestone.query.filter_by(active = True).first()
#        testbug1 = generate_bug(123456, 'first test bug', test_milestone)
#        db.session.add(testbug1)
#        db.session.commit()
#
#        # set up rss mock
#        testrss = FakeBodhiRss([testbug1.bugid])
#        monkeypatch.setattr(self.testsync, 'get_fixing_updates', testrss.get_fixing_updates)
#
#        # set up mock bodhi returns
#        testupdate1 = generate_updateinfo('magicfix-1.0-1.fc99', [testbug1.bugid, 234567])
#        testbodhi = FakeBodhiInterface(testupdate1)
#        monkeypatch.setattr(self.testsync, 'get_update', testbodhi.get_update)
#
#        self.testsync.sync_updates(test_milestone.release)
#
#        bugs = Bug.query.all()
#        assert len(bugs) == 1
#        bug = bugs[0]
#        assert len(bug.updates) == 1
#        assert bug.updates[0].title == testupdate1['title']
#
#    # add 1 bug to db, bugsync triggers full sync for that bug


